<?php
require_once('vendor/flexi/flexi.php');
require_once 'lib/include/reiter.inc.php';
class BenutzerStatistikPlugin extends AbstractStudIPSystemPlugin
{
	var $user;
	var $permissions;
	var $template_factory;
	var $db;
	var $tabs = array(
		'show'=>'Gesamt',
		'today'=>'Heute',
		'yesterday'=>'Gestern',
		'monthly'=>'Monatlich'
	);

	function BenutzerStatistikPlugin()
	{
		parent::AbstractStudIPSystemPlugin();

		$this->user = $this->getUser();
		$this->permissions = $this->user->getPermission();
		if ($this->permissions->hasRootPermission())
		{
			$nav = new PluginNavigation();
			$nav->setDisplayname($this->getPluginname());
			$this->setNavigation($nav);

			$this->setPluginiconname('images/stats.gif');
		}
		$this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
		$this->db = new DB_Seminar;
	}

	function getPluginname()
	{
		return _('Benutzer-Statistik');
	}

	function hasBackgroundTasks()
	{
		return true;
	}

	function doBackgroundTasks()
	{
		static $hit_once = false;

		if (!empty($_POST['sniffer_ping']))
		{
			$this->StoreHit();
		}
		elseif ($hit_once)
		{
			return;
		}

		$user_agent = empty($_SERVER['HTTP_USER_AGENT']) ? '?' : $_SERVER['HTTP_USER_AGENT'];

		$this->db->queryf("UPDATE user_statistics SET hits=hits+1 WHERE ip=INET_ATON('%s') AND user_agent='%s' AND hash=MD5(CONCAT('%s', '%s')) AND daystamp=NOW()", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());
		if ($this->db->affected_rows()==0)
		{
			$this->db->queryf("INSERT INTO user_statistics (ip, user_agent, daystamp, hash) VALUES (INET_ATON('%s'), '%s', NOW(), MD5(CONCAT('%s', '%s')))", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());
		}

		$this->db->queryf("SELECT COUNT(*) AS quantity FROM user_statistics WHERE ip=INET_ATON('%s') AND user_agent='%s' AND daystamp=NOW() AND hash=MD5(CONCAT('%s', '%s')) AND (hits>5 OR javascript=1)", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());
		if (!$this->db->next_record() or $this->db->f('quantity')==0)
		{
			$uri = 'http'.($_SERVER['SERVER_PORT']!=80?'s':'').'://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['REQUEST_URI']).'/index.php';

			$template = $this->template_factory->open('sniffer');
			$template->set_attribute('link', $uri);
			echo $template->render();
		}
		$this->db->free();

		$hit_once = true;
	}

	function StoreHit()
	{
		ob_end_clean();

		$user_agent = empty($_SERVER['HTTP_USER_AGENT']) ? '?' : $_SERVER['HTTP_USER_AGENT'];
		$this->db->queryf("UPDATE user_statistics SET javascript=1, screen_width=%d, screen_height=%d WHERE ip=INET_ATON('%s') AND user_agent='%s' AND hash=MD5(CONCAT('%s', '%s')) AND daystamp=NOW()", $_POST['width'], $_POST['height'], $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());

		die;
	}

	function displayHeader()
	{
		$structure = array();
		foreach ($this->tabs as $cmd=>$title)
		{
			$structure[$cmd] = array(
				'topKat'=>'',
				'name'=>_($title),
				'link'=>PluginEngine::getLink($this, array(), $cmd),
				'active'=>($cmd==$_REQUEST['cmd'])
			);
		}

		$reiter = new reiter;
		$reiter->create($structure, $_REQUEST['cmd']);
		echo '<br/>';
	}

	function actionshow()
	{
		$this->displayHeader();

		$template = $this->template_factory->open('stats');
		$template->set_attribute('stats', $this->getStatistics());
		$template->set_attribute('link', PluginEngine::getLink($this, array()).'&agent=');
		echo $template->render();
	}

	function actiontoday()
	{
		$this->displayHeader();

		$template = $this->template_factory->open('stats');
		$template->set_attribute('stats', $this->getStatistics(time()));
		$template->set_attribute('link', PluginEngine::getLink($this, array()).'&agent=');
		echo $template->render();
	}

	function actionyesterday()
	{
		$this->displayHeader();

		$template = $this->template_factory->open('stats');
		$template->set_attribute('stats', $this->getStatistics(time()-24*60*60));
		$template->set_attribute('link', PluginEngine::getLink($this, array()).'&agent=');
		echo $template->render();
	}

	function actionmonthly()
	{
		$month_names = array(
			1=>_('Januar'),
			2=>_('Februar'),
			3=>_('Mrz'),
			4=>_('April'),
			5=>_('Mai'),
			6=>_('Juni'),
			7=>_('Juli'),
			8=>_('August'),
			9=>_('September'),
			10=>_('Oktober'),
			11=>_('November'),
			12=>_('Dezember')
		);

		$this->displayHeader();

		if (empty($_REQUEST['month']))
		{
			$month = date('n');
			$year = date('Y');
		}
		else
		{
			list($month, $year) = explode('-', $_REQUEST['month']);
		}

		$months = array();
		$this->db->query("SELECT DISTINCT DATE_FORMAT(daystamp, '%m-%Y') AS month_stamp FROM user_statistics ORDER BY month_stamp ASC");
		while ($this->db->next_record())
		{
			list($m, $y) = explode('-', $this->db->f('month_stamp'));
			$months[$this->db->f('month_stamp')] = $month_names[$m].' '.$y;
		}
		$this->db->free();

		$stats = array();
		$this->db->queryf("SELECT DATE_FORMAT(daystamp, '%%e') AS day, SUM(hits) AS hits, COUNT(hash) AS visits, COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp", mktime(0,0,0,$month,1,$year), mktime(0,0,0,$month+1,1,$year)-1);
		while ($this->db->next_record())
		{
			$stats[$this->db->f('day')] = array(
				'hits'=>$this->db->f('hits'),
				'visits'=>$this->db->f('visits'),
				'headcount'=>$this->db->f('headcount')
			);
		}
		$this->db->free();

		$max = array();
		$totals = array();
		foreach ($stats as $stat)
		{
			foreach ($stat as $index=>$value)
			{
				if (empty($totals[$index]))
				{
					$totals[$index] = 0;
				}
				$totals[$index] += $value;

				if (empty($max[$index]))
				{
					$max[$index] = 0;
				}
				$max[$index] = max($max[$index], $value);
			}
		}

		$template = $this->template_factory->open('monthly');
		$template->set_attribute('months', $months);
		$template->set_attribute('month_stamp', mktime(0,0,0, $month, 1, $year));
		$template->set_attribute('stats', $stats);
		$template->set_attribute('max', $max);
		$template->set_attribute('totals', $totals);
		echo $template->render();
	}

	function getStatistics($date=null, $end_date=null)
	{
		$query_append = '';
		if ($date!==null and $end_date===null)
		{
			$query_append .= " daystamp=DATE(FROM_UNIXTIME(".$date."))";
		}
		elseif ($date!==null and $end_date!==null)
		{
			$query_append .= " daystamp BETWEEN DATE(FROM_UNIXTIME(".$date.")) AND DATE(FROM_UNIXTIME(".$end_date."))";
		}

		$query = "SELECT COUNT(DISTINCT hash) AS headcount, DATE_FORMAT(MIN(daystamp), '%%d.%%m.%%Y') AS start_date, DATE_FORMAT(MAX(daystamp), '%%d.%%m.%%Y') AS end_date, COUNT(*) AS visits, SUM(hits) AS hits, SUM(javascript)*100/COUNT(*) AS javascript_percentage FROM user_statistics";
		if (!empty($query_append))
		{
			$query .= " WHERE".$query_append;
		}
		$this->db->queryf($query);
		$this->db->next_record();
		$stats = array(
			'date'=> ($date!==null and $end_date===null)
				? date('d.m.Y', $date)
				: array(
					'start'=>$this->db->f('start_date'),
					'end'=>$this->db->f('end_date')
				)
			,
			'visits'=>$this->db->f('visits'),
			'headcount'=>$this->db->f('headcount'),
			'hits'=>$this->db->f('hits'),
			'javascript_percentage'=>$this->db->f('javascript_percentage'),
			'screen_sizes'=>array(),
			'user_agents'=>array()
		);
		$this->db->free();

		$query = "SELECT COUNT(*) AS quantity, screen_width, screen_height FROM user_statistics WHERE screen_width IS NOT NULL AND screen_height IS NOT NULL ".(empty($query_append)?'':" AND".$query_append)."GROUP BY screen_width, screen_height ORDER BY quantity DESC LIMIT 10";
		$this->db->queryf($query);
		while ($this->db->next_record())
		{
			array_push($stats['screen_sizes'], array(
				'width'=>$this->db->f('screen_width'),
				'height'=>$this->db->f('screen_height'),
				'percentage'=>$this->db->f('quantity')*100/$stats['visits']
			));
		}
		$this->db->free();

		$query = "SELECT user_agent, COUNT(*) AS quantity FROM user_statistics ".(empty($query_append)?'':" WHERE".$query_append)."GROUP BY user_agent";
		$this->db->queryf($query);
		$user_agents = array();
		while ($this->db->next_record())
		{
			array_push($user_agents, array(
				'user_agent'=>$this->db->f('user_agent'),
				'quantity'=>$this->db->f('quantity')
			));
		}
		$this->db->free();

		$stats['user_agents'] = $this->parseUserAgents($user_agents);

		$query = "SELECT AVG(hits) AS hits FROM user_statistics";
		if (!empty($query_append))
		{
			$query .= " WHERE".$query_append;
		}
		$this->db->query($query);
		$stats['avg_hits'] = $this->db->next_record() ? $this->db->f('hits') : 0;

		return $stats;
	}

	function parseUserAgents($user_agents)
	{
		$result = array();
		foreach ($user_agents as $user_agent)
		{
			$quantity = $user_agent['quantity'];
			$agent = $user_agent['user_agent'];
			$version = '?';
			if (preg_match('~(\d+(?:\.\d+)*)\s+(Safari)~i', $agent, $matches))
			{
				$agent = $matches[2];
				$version = $matches[1];
			}
			elseif (preg_match('~(Safari|SeaMonkey|IceWeasel|Konqueror|Netscape|Opera|Firefox|MSIE)[ /]([^;\s]+)(?:;|\s|$)~i', $agent, $matches))
			{
				$agent = $matches[1];
				$version = $matches[2];
			}
			else
			{
				$version = $agent;
				$agent = _('unbekannt');
			}

			$hash = md5($agent);
			if (!empty($result[$hash]))
			{
				$result[$hash]['quantity'] += $quantity;
				if (empty($result[$hash]['versions'][$version]))
				{
					$result[$hash]['versions'][$version] = 0;
				}
				$result[$hash]['versions'][$version] += $quantity;
			}
			else
			{
				$result[$hash] = array(
					'name'=>$agent,
					'versions'=>array(
						$version=>$quantity
					),
					'quantity'=>$quantity
				);
			}
		}

		foreach ($result as $index=>$item)
		{
			uasort($item['versions'], create_function('$a, $b', 'return $b-$a;'));
			$result[$index] = $item;
		}
		usort($result, create_function('$a, $b', 'return $b["quantity"]-$a["quantity"];'));

		return array_slice($result, 0, 10);
	}

	function show()
	{
		return $this->actionshow();
	}

	function today()
	{
		return $this->actiontoday();
	}

	function yesterday()
	{
		return $this->actionyesterday();
	}

	function monthly()
	{
		return $this->actionmonthly();
	}
}
?>